'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var theming = require('theming');
var _extends = _interopDefault(require('@babel/runtime/helpers/extends'));
var _inheritsLoose = _interopDefault(require('@babel/runtime/helpers/inheritsLoose'));
var React = require('react');
var React__default = _interopDefault(React);
var PropTypes = _interopDefault(require('prop-types'));
var defaultJss = require('jss');
var defaultJss__default = _interopDefault(defaultJss);
var preset = _interopDefault(require('jss-preset-default'));
var _createClass = _interopDefault(require('@babel/runtime/helpers/createClass'));
var _objectWithoutPropertiesLoose = _interopDefault(require('@babel/runtime/helpers/objectWithoutPropertiesLoose'));
var hoistNonReactStatics = _interopDefault(require('hoist-non-react-statics'));
var warning = _interopDefault(require('tiny-warning'));

var JssContext = React__default.createContext({
  sheetOptions: {},
  disableStylesGeneration: false
});

var memoize = function memoize(fn) {
  var lastArgs;
  var lastResult;
  return function () {
    for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    if (Array.isArray(lastArgs) && args.length === lastArgs.length) {
      var isSame = true;

      for (var i = 0; i < args.length; i++) {
        if (args[i] !== lastArgs[i]) {
          isSame = false;
        }
      }

      if (isSame) {
        return lastResult;
      }
    }

    lastArgs = args;
    lastResult = fn.apply(void 0, args);
    return lastResult;
  };
};

/* eslint-disable react/require-default-props, react/no-unused-prop-types */

var JssProvider =
/*#__PURE__*/
function (_Component) {
  _inheritsLoose(JssProvider, _Component);

  function JssProvider() {
    var _this;

    for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    _this = _Component.call.apply(_Component, [this].concat(args)) || this;
    _this.managers = {};
    _this.createContext = memoize(function (outerContext, props) {
      var registry = props.registry,
          classNamePrefix = props.classNamePrefix,
          jss = props.jss,
          generateId = props.generateId,
          disableStylesGeneration = props.disableStylesGeneration,
          media = props.media; // Clone the outer context

      var context = _extends({}, outerContext);

      if (registry) {
        context.registry = registry; // This way we identify a new request on the server, because user will create
        // a new Registry instance for each.

        if (registry !== _this.registry) {
          // We reset managers because we have to regenerate all sheets for the new request.
          _this.managers = {};
          _this.registry = registry;
        }
      }

      context.managers = _this.managers;

      if (generateId) {
        context.sheetOptions.generateId = generateId;
      } else if (!context.sheetOptions.generateId) {
        if (!_this.generateId) {
          _this.generateId = defaultJss.createGenerateId();
        }

        context.sheetOptions.generateId = _this.generateId;
      } // Merge the classname prefix


      if (classNamePrefix) {
        context.sheetOptions.classNamePrefix = (context.sheetOptions.classNamePrefix || '') + classNamePrefix;
      }

      if (media !== undefined) {
        context.sheetOptions.media = media;
      }

      if (jss) {
        context.jss = jss;
      }

      if (disableStylesGeneration !== undefined) {
        context.disableStylesGeneration = disableStylesGeneration;
      }

      return context;
    });
    _this.generateId = void 0;
    _this.registry = void 0;

    _this.renderProvider = function (outerContext) {
      var children = _this.props.children; // $FlowFixMe

      var context = _this.createContext(outerContext, _this.props);

      return React__default.createElement(JssContext.Provider, {
        value: context
      }, children);
    };

    return _this;
  }

  var _proto = JssProvider.prototype;

  _proto.render = function render() {
    return React__default.createElement(JssContext.Consumer, null, this.renderProvider);
  };

  return JssProvider;
}(React.Component);

JssProvider.propTypes = {
  registry: PropTypes.instanceOf(defaultJss.SheetsRegistry),
  jss: PropTypes.instanceOf(defaultJss__default.constructor),
  generateId: PropTypes.func,
  classNamePrefix: PropTypes.string,
  disableStylesGeneration: PropTypes.bool,
  children: PropTypes.node.isRequired,
  media: PropTypes.string
};

var defaultJss$1 = defaultJss.create(preset());

var getDisplayName = (function (Component) {
  return Component.displayName || Component.name || 'Component';
});

function mergeClasses(baseClasses, additionalClasses) {
  var combinedClasses = _extends({}, baseClasses);

  for (var name in additionalClasses) {
    combinedClasses[name] = name in combinedClasses ? combinedClasses[name] + " " + additionalClasses[name] : additionalClasses[name];
  }

  return combinedClasses;
}

/**
 * Global index counter to preserve source order.
 * As we create the style sheet during componentWillMount lifecycle,
 * children are handled after the parents, so the order of style elements would
 * be parent->child. It is a problem though when a parent passes a className
 * which needs to override any childs styles. StyleSheet of the child has a higher
 * specificity, because of the source order.
 * So our solution is to render sheets them in the reverse order child->sheet, so
 * that parent has a higher specificity.
 */
var indexCounter = -Number.MAX_SAFE_INTEGER;
var managersCounter = 0;

var NoRenderer = function NoRenderer(props) {
  return props.children || null;
};

var sheetsMeta = new WeakMap();

var getStyles = function getStyles(styles, theme, displayName) {
  if (typeof styles !== 'function') {
    return styles;
  }

  process.env.NODE_ENV !== "production" ? warning(styles.length !== 0, "[JSS] <" + displayName + " />'s styles function doesn't rely on the \"theme\" argument. We recommend declaring styles as an object instead.") : void 0;
  return styles(theme);
};

var getSheetClasses = function getSheetClasses(sheet, dynamicRules) {
  if (!dynamicRules) {
    return sheet.classes;
  }

  var classes = {};
  var meta = sheetsMeta.get(sheet);

  if (!meta) {
    return sheet.classes;
  }

  for (var key in meta.themedStyles) {
    classes[key] = sheet.classes[key];

    if (key in dynamicRules) {
      classes[key] += " " + sheet.classes[dynamicRules[key].key];
    }
  }

  return classes;
};

var updateDynamicRules = function updateDynamicRules(props, _ref) {
  var dynamicRules = _ref.dynamicRules,
      sheet = _ref.sheet;

  if (!sheet) {
    return;
  } // Loop over each dynamic rule and update it
  // We can't just update the whole sheet as this has all of the rules for every component instance


  for (var key in dynamicRules) {
    sheet.update(dynamicRules[key].key, props);
  }
};

var removeDynamicRules = function removeDynamicRules(_ref2) {
  var dynamicRules = _ref2.dynamicRules,
      sheet = _ref2.sheet;

  if (!sheet) {
    return;
  } // Loop over each dynamic rule and remove the dynamic rule
  // We can't just remove the whole sheet as this has all of the rules for every component instance


  for (var key in dynamicRules) {
    sheet.deleteRule(dynamicRules[key].key);
  }
};

var addDynamicStyles = function addDynamicStyles(sheet) {
  var meta = sheetsMeta.get(sheet);

  if (!meta) {
    return undefined;
  }

  var rules = {}; // Loop over each dynamic rule and add it to the stylesheet

  for (var key in meta.dynamicStyles) {
    var name = key + "-" + meta.dynamicRuleCounter++;
    var rule = sheet.addRule(name, meta.dynamicStyles[key]);

    if (rule) {
      rules[key] = rule;
    }
  }

  return rules;
};
/**
 * HOC creator function that wrapps the user component.
 *
 * `withStyles(styles, [options])(Component)`
 */


function withStyles(styles, options) {
  var _ref3 = options || {},
      _ref3$index = _ref3.index,
      index = _ref3$index === void 0 ? indexCounter++ : _ref3$index,
      theming$$1 = _ref3.theming,
      injectTheme = _ref3.injectTheme,
      optionsJss = _ref3.jss,
      sheetOptions = _objectWithoutPropertiesLoose(_ref3, ["index", "theming", "injectTheme", "jss"]);

  var isThemingEnabled = typeof styles === 'function';
  var ThemeConsumer = theming$$1 && theming$$1.context.Consumer || theming.ThemeContext.Consumer;
  return function (InnerComponent) {
    if (InnerComponent === void 0) {
      InnerComponent = NoRenderer;
    }

    var displayName = getDisplayName(InnerComponent);
    var defaultClassNamePrefix = process.env.NODE_ENV === 'production' ? '' : displayName.replace(/\s/g, '-') + "-";
    var managerId = managersCounter++;
    var manager = new defaultJss.SheetsManager();
    var emptyTheme = {};

    var getTheme = function getTheme(props) {
      return isThemingEnabled && props.theme ? props.theme : emptyTheme;
    };

    var WithStyles =
    /*#__PURE__*/
    function (_Component) {
      _inheritsLoose(WithStyles, _Component);

      // $FlowFixMe
      function WithStyles(props) {
        var _this;

        _this = _Component.call(this, props) || this;
        _this.mergeClassesProp = memoize(function (sheetClasses, classesProp) {
          return classesProp ? mergeClasses(sheetClasses, classesProp) : sheetClasses;
        });
        _this.classNamePrefix = void 0;
        var contextSheetOptions = props.jssContext.sheetOptions;
        _this.classNamePrefix = (contextSheetOptions.classNamePrefix || '') + defaultClassNamePrefix;
        _this.state = _this.createState();

        _this.manage(props, _this.state);

        return _this;
      }

      var _proto = WithStyles.prototype;

      _proto.componentDidUpdate = function componentDidUpdate(prevProps, prevState) {
        updateDynamicRules(this.props, this.state);

        if (isThemingEnabled && this.props.theme !== prevProps.theme) {
          var newState = this.createState();
          this.manage(this.props, newState);
          this.unmanage(prevProps, prevState); // eslint-disable-next-line react/no-did-update-set-state

          this.setState(newState);
        }
      };

      _proto.componentWillUnmount = function componentWillUnmount() {
        this.unmanage(this.props, this.state);
      };

      _proto.getSheet = function getSheet() {
        var theme = getTheme(this.props);
        var sheet = this.manager.get(theme);

        if (sheet) {
          return sheet;
        }

        var themedStyles = getStyles(styles, theme, displayName);
        var dynamicStyles = defaultJss.getDynamicStyles(themedStyles);
        var contextSheetOptions = this.props.jssContext.sheetOptions;
        sheet = this.jss.createStyleSheet(themedStyles, _extends({}, sheetOptions, contextSheetOptions, {
          index: index,
          meta: displayName + ", " + (isThemingEnabled ? 'Themed' : 'Unthemed'),
          classNamePrefix: this.classNamePrefix,
          link: dynamicStyles !== null
        }));
        sheetsMeta.set(sheet, {
          dynamicStyles: dynamicStyles,
          themedStyles: themedStyles,
          dynamicRuleCounter: 0
        });
        this.manager.add(theme, sheet);
        return sheet;
      };

      _proto.manage = function manage(props, state) {
        var sheet = state.sheet;
        var registry = props.jssContext.registry;

        if (!sheet) {
          return;
        }

        updateDynamicRules(props, state);
        this.manager.manage(getTheme(props));

        if (registry) {
          registry.add(sheet);
        }
      };

      _proto.unmanage = function unmanage(props, state) {
        removeDynamicRules(state);
        this.manager.unmanage(getTheme(props));
      };

      _proto.createState = function createState() {
        if (this.props.jssContext.disableStylesGeneration) {
          return {
            classes: {}
          };
        }

        var sheet = this.getSheet(); // Those are dynamic rules which are used in this specific element only.

        var dynamicRules = addDynamicStyles(sheet);
        var classes = getSheetClasses(sheet, dynamicRules);
        var state = {
          sheet: sheet,
          classes: classes
        };
        if (dynamicRules) state.dynamicRules = dynamicRules;
        return state;
      };

      _proto.render = function render() {
        var _this$props = this.props,
            innerRef = _this$props.innerRef,
            jssContext = _this$props.jssContext,
            theme = _this$props.theme,
            classes = _this$props.classes,
            rest = _objectWithoutPropertiesLoose(_this$props, ["innerRef", "jssContext", "theme", "classes"]);

        var sheetClasses = this.state.classes;

        var props = _extends({}, rest, {
          classes: this.mergeClassesProp(sheetClasses, classes)
        });

        if (innerRef) props.ref = innerRef;
        if (injectTheme) props.theme = theme;
        return React__default.createElement(InnerComponent, props);
      };

      _createClass(WithStyles, [{
        key: "jss",
        get: function get() {
          return this.props.jssContext.jss || optionsJss || defaultJss$1;
        }
      }, {
        key: "manager",
        get: function get() {
          var managers = this.props.jssContext.managers; // If `managers` map is present in the context, we use it in order to
          // let JssProvider reset them when new response has to render server-side.

          if (managers) {
            if (!managers[managerId]) {
              managers[managerId] = new defaultJss.SheetsManager();
            }

            return managers[managerId];
          }

          return manager;
        }
      }]);

      return WithStyles;
    }(React.Component); // $FlowFixMe: Sadly there is no support for forwardRef yet


    WithStyles.displayName = "WithStyles(" + displayName + ")";
    WithStyles.defaultProps = _extends({}, InnerComponent.defaultProps);
    var JssContextSubscriber = React__default.forwardRef(function (props, ref) {
      return React__default.createElement(JssContext.Consumer, null, function (context) {
        if (isThemingEnabled || injectTheme) {
          return React__default.createElement(ThemeConsumer, null, function (theme) {
            return React__default.createElement(WithStyles, _extends({
              innerRef: ref,
              theme: theme
            }, props, {
              jssContext: context
            }));
          });
        }

        return React__default.createElement(WithStyles, _extends({
          innerRef: ref
        }, props, {
          jssContext: context
        }));
      });
    });
    JssContextSubscriber.displayName = 'JssContextSubscriber';
    JssContextSubscriber.InnerComponent = InnerComponent;
    return hoistNonReactStatics(JssContextSubscriber, InnerComponent);
  };
}

exports.ThemeProvider = theming.ThemeProvider;
exports.withTheme = theming.withTheme;
exports.createTheming = theming.createTheming;
exports.SheetsRegistry = defaultJss.SheetsRegistry;
exports.createGenerateId = defaultJss.createGenerateId;
exports.JssProvider = JssProvider;
exports.jss = defaultJss$1;
exports.JssContext = JssContext;
exports.default = withStyles;
